package com.malk.jisuanji.controller;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.malk.Util.UtilMap;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.server.common.McException;
import com.malk.server.common.McR;
import com.malk.server.dingtalk.DDR_New;
import com.malk.service.aliwork.YDClient;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 错误抛出与拦截详见CatchException
 */
@Slf4j
@RestController
@RequestMapping("/jisuanji")
public class HSController {

    @Autowired
    private YDClient ydClient;

    /**
     * 供应商数据查询
     */
    @PostMapping("yd/supplier")
    McR querySupplier(@RequestBody Map<String, String> data) {

        log.info("供应商数据查询, {}", data);
        McException.assertParamException_Null(data, "condition", "formUuid", "compId");
        List conditions = Arrays.asList(UtilMap.map("key, value, type, operator, componentName", data.get("compId"), data.get("condition"), "TEXT", "eq", "TextField"));
        YDParam ydParam = YDParam.builder()
                .formUuid(String.valueOf(data.get("formUuid")))
                .searchCondition(JSON.toJSONString(conditions))
                .build();
        DDR_New ddr_new = ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_list_all);
        McException.assertAccessException(ddr_new.getTotalCount() == 0, "代码无效, 不在供应商库");
        return McR.success(((List) ddr_new.getData()).get(0));
    }

    /**
     * 供应商下单数据
     */
    @PostMapping("yd/order-list")
    McR queryOrderStatus(@RequestBody Map<String, Map> data) {

        log.info("供应商下单数据, {}", data);
        querySupplier(data.get("supplier")); // 校验供应商信息
        Map param = data.get("order");
        List<Map> conditions = new ArrayList<>();
        conditions.add(UtilMap.map("key, value, type, operator, componentName", param.get("compId_code"), param.get("condition_code"), "TEXT", "eq", "TextField"));
        if (ObjectUtil.isNotNull(param.get("condition_date"))) {
            conditions.add(UtilMap.map("key, value, type, operator, componentName", param.get("compId_date"), param.get("condition_date"), "DOUBLE", "between", "DateField"));
        }
        YDParam ydParam = YDParam.builder()
                .formUuid(String.valueOf(param.get("formUuid")))
                .searchCondition(JSON.toJSONString(conditions))
                .build();
        DDR_New ddr_new = ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_search_process);
        return McR.success(ddr_new);
    }
}
