package com.malk.lichen.controller;

import com.alibaba.fastjson.JSON;
import com.malk.Util.UtilMap;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.server.common.McR;
import com.malk.service.aliwork.YDClient;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 错误抛出与拦截详见 CatchException
 */
@Slf4j
@RestController
@RequestMapping("/lichen")
@EnableAsync
public class LiChenController {

    /**** 力辰服务 ****/

    @Autowired
    private YDClient ydClient;

    /**
     * 查询实例详情
     */
    @GetMapping("form/{formInstId}")
    McR getInstanceById(@PathVariable String formInstId) {

        log.info("查询实例详情, {}", formInstId);

        YDParam ydParam = YDParam.builder()
                .appType("APP_BV3K6QNMINKYEDHQNI4N")
                .systemToken("CH766981WHM8OEVBEAKCY7MYB9SV3ORPLIQELS5")
                .formInstId(formInstId)
                .build();
        return McR.success(ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_id));
    }

    /**
     * 写入往来单位
     */
    @PostMapping("form/wldx")
    McR updateWangLaiDanWei(@RequestBody Map data) {

        log.info("写入往来单位, {}", data);

        List<Map> list = (List<Map>) data.get("data");
        for (Map record : list) {
            Map searchMap = UtilMap.map("textField_lgpz4fmf", record.get("FNumber"));
            YDParam ydParam = YDParam.builder()
                    .formUuid("FORM-A8666NA1404AITYX7V2NI9VHO2TK3TYOVYPGL0")
                    .searchFieldJson(JSON.toJSONString(searchMap))
                    .build();
            long total = ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_list).getTotalCount();
            if (total == 0) {
                ydParam.formDataJson = JSON.toJSONString(UtilMap.map("textField_lgpz6rn1, textField_lgpz4fmf, selectField_lgpz4fm9", "FName, FNumber, Type", record));
                ydClient.operateData(ydParam, YDConf.FORM_OPERATION.create);
            }
        }
        return McR.success();
    }

    /**
     * 写入付款申请
     */
    @SneakyThrows
    @PostMapping("form/fksq")
    McR updateFuKuanShenQing(@RequestBody Map data) {

        log.info("写入付款申请, {}", data);

        List<Map> list = (List<Map>) data.get("data");
        for (Map record : list) {
            String billNo = String.valueOf(record.get("FBillNo"));
            if (billNo.contains("FKSQ")) {
                Map searchMap = UtilMap.map("textField_lgov75ms", billNo);
                YDParam ydParam = YDParam.builder()
                        .appType("APP_BV3K6QNMINKYEDHQNI4N")
                        .systemToken("CH766981WHM8OEVBEAKCY7MYB9SV3ORPLIQELS5")
                        .formUuid("FORM-4V966QC142J8WYDD9PO74DD2QOME3ZG0MIQELK1")
                        .searchFieldJson(JSON.toJSONString(searchMap))
                        .build();
                long total = ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_list).getTotalCount();
                if (total == 0) {
                    Map formData = UtilMap.map("textField_lgov75ms, radioField_lguqzv36", billNo, "是");//单据编号
                    formData.put("textField_lglwgnl7", record.get("FCREATORID"));//创建人ID
                    formData.put("textField_lguqqubv", record.get("FCREATORNAME"));//创建人姓名
                    formData.put("textField_lgvpbhx0", record.get("FSETTLEORGID"));//结算组织
                    formData.put("", record.get("FPURCHASEORGID")); //采购组织
                    formData.put("textField_leqinlpz", record.get("FAPPLYORGID")); //申请组织
                    formData.put("dateField_leqinlqd", record.get("FDATE")); //申请日期
                    formData.put("selectField_lgvpbhwz", record.get("FBILLTYPEID"));//单据类型
                    formData.put("selectField_lgpzoio6", record.get("FCONTACTUNITTYPE"));//往来单位类型
                    formData.put("selectField_lexydxa5", record.get("FCONTACTUNIT"));//往来单位
                    formData.put("textField_lgpzoio7", record.get("FCONTACTUNIT_Id"));//往来单位编码
                    formData.put("", record.get("FBUSINESSTYPE"));//业务类型
                    formData.put("textField_leqinlq3", record.get("FPAYORGID"));//付款组织
                    formData.put("textField_lgvxe8q6", record.get("FRECTUNIT"));//收款单位
                    formData.put("selectField_lgvpbhx1", record.get("F_BQB_Combo"));//付款类型
                    formData.put("radioField_leqinlq7", record.get("F_BQB_FKDQ"));//付款地区
                    formData.put("selectField_lgvz3r3j", record.get("F_BQB_FKGS"));//付款公司
                    formData.put("textField_leqinlq2", record.get("FEXPENSEDEPTID"));//费用承担部门
                    List<Map> details = new ArrayList<>();
                    List<Map> dataTable = (List<Map>) JSON.parse(String.valueOf(record.get("DataTable")));
                    for (Map detail : dataTable) {
                        Map rowTable = UtilMap.map("selectField_lewl78sf", detail.get("FSETTLETYPEID"));//结算方式
                        rowTable.put("textField_lfw71laq", detail.get("FEACHBANKACCOUNT"));//对方银行账号
                        rowTable.put("numberField_leqinlqa", detail.get("FAPPLYAMOUNTFOR"));//申请付款金额
                        rowTable.put("dateField_lewl78sh", detail.get("FENDDATE"));//到期日
                        rowTable.put("dateField_lewl78sg", detail.get("FEXPECTPAYDATE"));//期望付款日期
                        rowTable.put("textField_lfw71lar", detail.get("FEACHCCOUNTNAME"));//对方账户名称
                        rowTable.put("textField_lfw71las", detail.get("FEACHBANKNAME"));//对方账户名称
                        rowTable.put("selectField_lgqbuu2d", detail.get("FCOSTID"));//费用项目
                        rowTable.put("textField_lgyosvjh", detail.get("FCOSTID_id"));//费用项目ID
                        rowTable.put("textField_leqinlqb", detail.get("FDescription"));//备注
                        details.add(rowTable);
                    }
                    formData.put("tableField_leqinlq8", details);
                    ydParam.formDataJson = JSON.toJSONString(formData);
                    ydParam.setProcessCode("TPROC--4V966QC142J8WYDD9PO74DD2QOME30H0MIQELL1");
                    ydClient.operateData(ydParam, YDConf.FORM_OPERATION.start);
                }
            }
        }
        return McR.success();
    }
}
