package com.malk.shanghaishiwei.service.impl;

import com.alibaba.fastjson.JSON;
import com.malk.Util.UtilMap;
import com.malk.shanghaishiwei.server.HTConf;
import com.malk.shanghaishiwei.server.HTR;
import com.malk.shanghaishiwei.service.HTClient;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * 上海市委_合同接口 [参数大写不敏感]
 */
@Service
@Slf4j
public class HTImplClient implements HTClient {

    /**
     * 获取采购合同类型列表
     */
    @Override
    public List<Map> listContractType() {
        Map body = UtilMap.map("SystemId, token", HTConf.SYSTEM_ID, HTConf.getToken());
        HTR htr = HTR.doPost("http://cw-shmecsfa.shec.edu.cn/CZZX/SFP_ContractApi/Contract/ListContractType", null, null, body);
        return (List<Map>) JSON.parse(htr.getData());
    }

    /**
     * 添加采购合同
     */
    @Override
    public boolean addPurchaseContact(Map body) {
        body.put("SystemId", HTConf.SYSTEM_ID);
        body.put("token", HTConf.getToken());
        return HTR.doPost("http://cw-shmecsfa.shec.edu.cn/CZZX/SFP_ContractApi/Contract/AddVenContract", null, null, body).isSuccess();
    }

    /**
     * 删除采购合同合同信息
     */
    @Override
    public boolean deletePurchaseContract(String origionContractcode) {
        Map body = UtilMap.map("SystemId, token, OrigionContractcode", HTConf.SYSTEM_ID, HTConf.getToken(), origionContractcode);
        return HTR.doPost("http://cw-shmecsfa.shec.edu.cn/CZZX/SFP_ContractApi/Contract/DelVenContract", null, null, body).isSuccess();
    }

    /**
     * 添加采购合同条款信息
     */
    @Override
    public boolean addTermsContract(Map body) {
        body.put("SystemId", HTConf.SYSTEM_ID);
        body.put("token", HTConf.getToken());
        return HTR.doPost("http://cw-shmecsfa.shec.edu.cn/CZZX/SFP_ContractApi/Contract/AddVenAgreement", null, null, body).isSuccess();
    }

    /**
     * 删除采购合同条款信息
     */
    @Override
    public boolean deleteTermsContract(String origionContractCode, String datetime, int agreementType) {
        Map body = UtilMap.map("SystemId, token, origionContractCode, datetime, agreementType", HTConf.SYSTEM_ID, HTConf.getToken(), origionContractCode, datetime, agreementType);
        return HTR.doPost("http://cw-shmecsfa.shec.edu.cn/CZZX/SFP_ContractApi/Contract/DelVenAgreement", null, null, body).isSuccess();
    }

    /**
     * 添加采购合同附件信息
     */
    @Override
    public boolean addAttachmentContract(Map body) {
        body.put("SystemId", HTConf.SYSTEM_ID);
        body.put("token", HTConf.getToken());
        return HTR.doPost("http://cw-shmecsfa.shec.edu.cn/CZZX/SFP_ContractApi/Contract/AddScanning", null, null, body).isSuccess();
    }

    /**
     * 删除采购合同附件
     */
    @Override
    public boolean deleteAttachmentContract(String origionContractCode, String fileType, String fileName) {
        Map body = UtilMap.map("SystemId, token, OrigionContractcode, fileType, fileName, contractType", HTConf.SYSTEM_ID, HTConf.getToken(), origionContractCode, fileType, fileName, 1);
        return HTR.doPost("http://cw-shmecsfa.shec.edu.cn/CZZX/SFP_ContractApi/Contract/DelScanning", null, null, body).isSuccess();
    }
}
