package com.malk.xiding.schedule;

import com.malk.Util.UtilDateTime;
import com.malk.xiding.server.XDConf;
import com.malk.xiding.service.XDService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import java.util.Date;

/**
 * @EnableScheduling 开启定时任务 [配置参考McScheduleTask]
 */
@Slf4j
@Configuration
@EnableScheduling
@ConditionalOnProperty(name = {"spel.scheduling"})
public class XDScheduleTask {

    @Autowired
    private XDService xdService;

    /**
     * [定时同步审批单]
     * 每天8点, 15点: 流程搜索按照发起和结束时间, 兼容性同步15内审批单
     */
    @Scheduled(cron = "0 0 8,15 1/1 * ?")
    public void Time_Period_1() {
        try {
            log.info("###### [XD]同步审批单开始 ######");
            long endTime = new Date().getTime();
            long startTime = endTime - 15 * 24 * 60 * 60 * 1000L;
            xdService.syncApprove(startTime, endTime, XDConf.TMP_CODE_CC, "出差");
            xdService.syncApprove(startTime, endTime, XDConf.TMP_CODE_WC, "外出");
        } catch (Exception e) {
            log.error(e.getMessage(), e); // 记录错误日志
        }
        log.info("###### [XD]同步审批单结束 ######");
    }

    /**
     * [频率为每10分钟查询一次, 同步当天全量时候, 过滤重复考勤, 避免钉钉调用次数过高]
     */
    @Scheduled(cron = "0 9/10 9-10,16-23,0-4  1/1 * ?")
    public void Time_Period_2() {
        log.info("###### [XD]同步考勤开始 ######");
        Date now = new Date();
        Date sTime = UtilDateTime.parseDateTime(UtilDateTime.formatDate(now) + " 00:00:00");
        Date eTime = UtilDateTime.parseDateTime(UtilDateTime.formatDate(now) + " 23:59:59");
        try {
            xdService.syncAttendance(0, sTime, eTime);
        } catch (Exception e) {
            log.error(e.getMessage(), e); // 记录错误日志
        }
        log.info("###### [XD]同步考勤结束 ######");
    }
}