package com.malk.xintianlong.controller;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.malk.Util.UtilMap;
import com.malk.base.BaseDto;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.server.aliwork.YDR;
import com.malk.server.common.McException;
import com.malk.server.common.McR;
import com.malk.server.dingtalk.DDConf;
import com.malk.server.dingtalk.DDFormComponentDto;
import com.malk.server.dingtalk.DDR_New;
import com.malk.service.aliwork.YDClient;
import com.malk.service.dingtalk.DDClient;
import com.malk.service.dingtalk.DDClient_Workflow;
import com.malk.xintianlong.repository.dao.HSApproveRecordDao;
import com.malk.xintianlong.repository.dao.HSViewDao;
import com.malk.xintianlong.repository.entity.HSApproveRecordPo;
import com.malk.xintianlong.server.HSConf;
import com.malk.xintianlong.service.HSService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

/**
 * 错误抛出与拦截详见CatchException
 */
@Slf4j
@RestController
@RequestMapping
public class HSController {

    @Autowired
    private DDClient ddClient;

    @Autowired
    private DDClient_Workflow ddClient_workflow;

    @Autowired
    private HSApproveRecordDao approveRecordDao;

    @Autowired
    private HSService hsService;

    /**
     * 推送申购单审批
     */
    @PostMapping("dd/sgd")
    McR purchaseApprove(@RequestBody Map data, HttpServletRequest request) {

        log.info("推送申购单审批, {}", data);
        hsService.validateAuthorization(request, String.valueOf(data.get("orderCode")), String.valueOf(data.get("corpType")));

        // 组件数据格式化
        data.put("corpTypeName", HSConf.SUB_CORP_DD.get(data.get("corpType")));
        Map ruleForm = UtilMap.map("corpTypeName, orderCode, proposerDate, purchaseType, goods", "公司主体, 请购单号, 请购日期, 采购类型, 采购明细");
        Map ruleDetail = UtilMap.map("name, size, num, remark", "物品名称, 规格, 数量, 备注");

        // 推送钉钉审批
        List<Map> formValues = DDFormComponentDto.formatComponentValues(data, ruleForm, ruleDetail);
        String userId = String.valueOf(data.get("proposerCode"));
        Map extInfo = UtilMap.map("dept_id", DDConf.TOP_DEPARTMENT);
        String processInstanceId = ddClient_workflow.doProcessInstances(ddClient.getAccessToken(), userId, HSConf.PURCHASE_CODE_DD, formValues, extInfo);

        // 保存推送记录
        HSApproveRecordPo po = JSON.parseObject(JSON.toJSONString(data), HSApproveRecordPo.class);
        po.setProcessInstanceId(processInstanceId);
        po.setRemark("采购申请单");
        po.setFormType("PU001");
        approveRecordDao.save(po);

        return McR.success(po);
    }

    /**
     * 推送结算单审批
     */
    @PostMapping("dd/jsd")
    McR settleApprove(@RequestBody Map data, HttpServletRequest request) {

        log.info("推送结算单审批, {}", data);
        hsService.validateAuthorization(request, String.valueOf(data.get("orderCode")), String.valueOf(data.get("corpType")));

        // 组件数据格式化
        data.put("corpTypeName", HSConf.SUB_CORP_DD.get(data.get("corpType")));
        data.put("salesmanCodeDing", JSON.toJSONString(Arrays.asList(data.get("salesmanCode"))));
        if (StringUtils.isNotBlank(String.valueOf(data.get("transportationCode2")))) {
            data.put("transportationCode", data.get("transportationCode") + "/" + data.get("transportationCode2"));
        }
        if (StringUtils.isNotBlank(String.valueOf(data.get("proposerDate2")))) {
            data.put("proposerDate", data.get("proposerDate") + "/" + data.get("proposerDate2"));
        }
        Map ruleForm = UtilMap.map("corpTypeName, orderCode, proposerDate, salesmanCodeDing", "所属公司, 结算单号, 出货日期, 业务员");
        ruleForm.put("taskCode", "指定书号");
        ruleForm.put("purchaseCorp", "要货单位");
        ruleForm.put("invoiceNumber", "发票号");
        ruleForm.put("productiionName", "产品名称");
        ruleForm.put("transportationCode", "调运单号");
        ruleForm.put("processingCharges", "加工费详情");
        ruleForm.put("totalPrice", "实收加工费(元)");

        Map ruleDetail = UtilMap.map("casting", UtilMap.map("taskCode, groupCode, size, count, price, total, supplier, remark", "指定书号, 组号, 品名/规格, 数量(米), 单价, 金额（元）, 供坯商, 备注"));
        ruleForm.put("casting", "投坯详情");

        ruleDetail.put("shipment", UtilMap.map("taskCode, groupCode, constructCode, size, num, count, price, total, priceSpread, remark", "指定书号, 组号, 合同号, 品名/规格, 件数, 数量(米), 单价, 金额（元）, 辅料差价(元), 备注"));
        ruleForm.put("shipment", "成品出货详情");

        ruleDetail.put("residue", UtilMap.map("taskCode, groupCode, size, num, count, price, total, otherPrice, remark", "指定书号, 组号, 品名/规格, 件数, 数量(米), 单价, 金额（元）, 其他金额(元), 备注"));
        ruleForm.put("residue", "余坯款");

        // 推送钉钉审批
        List<Map> formValues = DDFormComponentDto.formatComponentValues(data, ruleForm, ruleDetail);

        String userId = String.valueOf(data.get("proposerCode"));
        Map extInfo = UtilMap.map("dept_id", DDConf.TOP_DEPARTMENT);
        String processInstanceId = ddClient_workflow.doProcessInstances(ddClient.getAccessToken(), userId, HSConf.SETTLE_CODE_DD, formValues, extInfo);

        // 保存推送记录
        HSApproveRecordPo po = JSON.parseObject(JSON.toJSONString(data), HSApproveRecordPo.class);
        po.setProcessInstanceId(processInstanceId);
        po.setRemark("结算申请单");
        po.setFormType("TS001");
        approveRecordDao.save(po);

        return McR.success(po);
    }

    /**
     * 推送指定书审批
     */
    @PostMapping("dd/zds")
    McR assignmentLetter(@RequestBody Map data, HttpServletRequest request) {

        data.put("test", "是");
        log.info("推送指定书审批, {}", data);
        hsService.validateAuthorization(request, String.valueOf(data.get("orderCode")), String.valueOf(data.get("corpType")));

        // 组件数据格式化
        String userId = String.valueOf(data.get("proposerCode"));
        String orderCode = String.valueOf(data.get("orderCode"));
        data.put("corpTypeName", HSConf.SUB_CORP_DD.get(data.get("corpType")));
        data.put("proposerCodeDing", JSON.toJSONString(Arrays.asList(userId)));
        data.put("salesmanCodeDing", JSON.toJSONString(Arrays.asList(data.get("salesmanCode"))));
        Map detry = ddClient.uploadFileFormUrl(ddClient.getAccessToken(), userId, HSConf.getAttachmentUrl(orderCode, String.valueOf(data.get("corpType"))), orderCode);
        data.put("attachmentId", DDFormComponentDto.formatAttachment(detry));

        // 推送钉钉审批
        Map ruleForm = UtilMap.map("corpTypeName, orderCode, proposerCodeDing, proposerDate, consignee, salesmanCodeDing, attachmentId, test", "公司主体, 指定书号, 制单人, 制单日期, 要货单位, 业务员, 附件, 单选框");
        List<Map> formValues = DDFormComponentDto.formatComponentValues(data, ruleForm, null);
        String processInstanceId = ddClient_workflow.doProcessInstances(ddClient.getAccessToken(), userId, HSConf.PRODUCT_CODE_DD, formValues, UtilMap.map("dept_id", DDConf.TOP_DEPARTMENT));

        // 保存推送记录
        HSApproveRecordPo po = JSON.parseObject(JSON.toJSONString(data), HSApproveRecordPo.class);
        po.setProcessInstanceId(processInstanceId);
        po.setRemark("生产任务指定书");
        po.setFormType("SD001");
        approveRecordDao.save(po);

        return McR.success(po);
    }

    @Autowired
    HSApproveRecordDao hsApproveRecordDao;

    /**
     * 钉钉审批回调 [连接器-同意] - 废弃
     */
//    @PostMapping("dd/approve")
    McR purchaseApproveCallback(@RequestBody Map<String, String> data) {

        log.info("###### [HS]开始同步审批中记录 ######");
        List<HSApproveRecordPo> list = hsApproveRecordDao.findAllByApproveResult("process");
        log.info("同步数量, {}", list.size());
        for (HSApproveRecordPo po : list) {
            log.info("xxx, {}", po);
            Map rsp = ddClient_workflow.getProcessInstanceId(ddClient.getAccessToken(), po.getProcessInstanceId());
            if (Arrays.asList("RUNNING", "NEW").contains(rsp.get("status"))) {
                continue;
            }
            if (Arrays.asList("TERMINATED", "CANCELED").contains(rsp.get("status"))) {
                hsService.syncApproveResult2Hs(po.getProcessInstanceId(), "terminated");
            }
            if (StringUtils.isNotBlank(String.valueOf(rsp.get("result")))) {
                hsService.syncApproveResult2Hs(po.getProcessInstanceId(), "agree".equals(String.valueOf(rsp.get("result"))) ? "agree" : "refuse");
            }
        }

//        log.info("申购单审批回调, {}", data);
//        McException.assertParamException_Null(data, "processInstanceId", "approveResult");
//        String processInstanceId = data.get("processInstanceId");
//        return McR.success(hsService.syncApproveResult2Hs(processInstanceId, data.get("approveResult")));
        return McR.success();
    }

    /**
     * 查询钉钉审批结果
     */
    @PostMapping("dd/status")
    McR queryApproveResult(@RequestBody Map<String, List<String>> data) {

        McException.assertParamException_Null(data, "orders");
        return McR.success(approveRecordDao.findAllByOrderCodeIn(data.get("orders")));
    }

    /**
     * 撤销审批中单据
     */
    @PostMapping("dd/terminate")
    McR terminateApproveResult(@RequestBody Map<String, String> data) {

        McException.assertParamException_Null(data, "orderCode", "corpType");
        HSApproveRecordPo po = approveRecordDao.findByOrderCodeAndApproveResultAndCorpType(data.get("orderCode"), "process", data.get("corpType"));
        McException.assertAccessException(ObjectUtil.isNull(po), "该单号下, 没有审批中的单据!");
        ddClient_workflow.terminateRunningApprove(ddClient.getAccessToken(), po.getProcessInstanceId(), true, "环思取消送审", null);
        approveRecordDao.updateApproveResult(po.getProcessInstanceId(), "terminated", "环思取消送审");
        po.setApproveResult("terminated");
        return McR.success(po);
    }

    @Autowired
    private YDClient ydClient;

    /**
     * 供应商数据查询
     */
    @PostMapping("yd/supplier")
    McR querySupplier(@RequestBody Map<String, String> data) {

        log.info("供应商数据查询, {}", data);
        McException.assertParamException_Null(data, "condition", "formUuid", "compId");
        List conditions = Arrays.asList(UtilMap.map("key, value, type, operator, componentName", data.get("compId"), data.get("condition"), "TEXT", "eq", "TextField"));
        YDParam ydParam = YDParam.builder()
                .formUuid(String.valueOf(data.get("formUuid")))
                .searchCondition(JSON.toJSONString(conditions))
                .build();
        DDR_New ddr_new = ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_list_all);
        McException.assertAccessException(ddr_new.getTotalCount() == 0, "代码无效, 不在供应商库");
        return McR.success(((List) ddr_new.getData()).get(0));
    }

    /**
     * 供应商下单数据
     */
    @PostMapping("yd/order-list")
    McR queryOrderStatus(@RequestBody Map<String, Map> data) {

        log.info("供应商下单数据, {}", data);
        querySupplier(data.get("supplier")); // 校验供应商信息
        Map param = data.get("order");
        List<Map> conditions = new ArrayList<>();
        conditions.add(UtilMap.map("key, value, type, operator, componentName", param.get("compId_code"), param.get("condition_code"), "TEXT", "eq", "TextField"));
        if (ObjectUtil.isNotNull(param.get("condition_date"))) {
            conditions.add(UtilMap.map("key, value, type, operator, componentName", param.get("compId_date"), param.get("condition_date"), "DOUBLE", "between", "DateField"));
        }
        YDParam ydParam = YDParam.builder()
                .formUuid(String.valueOf(param.get("formUuid")))
                .searchCondition(JSON.toJSONString(conditions))
                .build();
        DDR_New ddr_new = ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_search_process);
        return McR.success(ddr_new);
    }


    @Autowired
    private HSViewDao hsOrderPlanDao;

    /**
     * 查询生产计划
     */
    @PostMapping("view/queryOrderPlan")
    McR queryOrderPlan(@RequestBody Map data) {

        log.info("查询生产计划, {}", data);

        String sOrderNo = UtilMap.getString(data, "sOrderNo");
        Pageable pageable = PageRequest.of(UtilMap.getInt(data, "page") - 1, UtilMap.getInt(data, "size"));

        Page page;
        if (UtilMap.isNotBlankString(data, "startTime") && UtilMap.isNotBlankString(data, "endTime")) {
            Date sTime = new Date(UtilMap.getLong(data, "startTime"));
            Date eTime = new Date(UtilMap.getLong(data, "endTime"));
            page = hsOrderPlanDao.queryOrderPlan(sTime, eTime, sOrderNo, pageable);
        } else if (UtilMap.isNotBlankString(data, "startTime")) {
            Date sTime = new Date(UtilMap.getLong(data, "startTime"));
            page = hsOrderPlanDao.queryOrderPlan_start(sTime, sOrderNo, pageable);
        } else if (UtilMap.isNotBlankString(data, "endTime")) {
            Date eTime = new Date(UtilMap.getLong(data, "endTime"));
            page = hsOrderPlanDao.queryOrderPlan_end(eTime, sOrderNo, pageable);
        } else {
            page = hsOrderPlanDao.queryOrderPlan(sOrderNo, pageable);
        }
        Map result = YDR.formatPage(page);

        return McR.success(BaseDto.jsonFormatDateTime(result));

    }

    /**
     * 查询成品库存
     */
    @PostMapping("view/queryStoreWarehouse")
    McR queryStoreWarehouse(@RequestBody Map data) {

        log.info("查询成品库存, {}", data);
        String sStoreInNo = UtilMap.getString(data, "sStoreInNo");
        Pageable pageable = PageRequest.of(UtilMap.getInt(data, "page") - 1, UtilMap.getInt(data, "size"));

        Page page;
        if (UtilMap.isNotBlankString(data, "startTime") && UtilMap.isNotBlankString(data, "endTime")) {
            Date sTime = new Date(UtilMap.getLong(data, "startTime"));
            Date eTime = new Date(UtilMap.getLong(data, "endTime"));
            page = hsOrderPlanDao.queryStoreWarehouse(sTime, eTime, sStoreInNo, pageable);
        } else if (UtilMap.isNotBlankString(data, "startTime")) {
            Date sTime = new Date(UtilMap.getLong(data, "startTime"));
            page = hsOrderPlanDao.queryStoreWarehouse_start(sTime, sStoreInNo, pageable);
        } else if (UtilMap.isNotBlankString(data, "endTime")) {
            Date eTime = new Date(UtilMap.getLong(data, "endTime"));
            page = hsOrderPlanDao.queryStoreWarehouse_end(eTime, sStoreInNo, pageable);
        } else {
            page = hsOrderPlanDao.queryStoreWarehouse(sStoreInNo, pageable);
        }
        Map result = YDR.formatPage(page);

        return McR.success(BaseDto.jsonFormatDateTime(result));
    }


    @PostMapping("test")
    McR test() {

//        approveRecordDao.save(HSApproveRecordPo.builder()
//                .corpType("01")
//                .processInstanceId("test1")
//                .formType("ts")
//                .orderCode("xx")
//                .approveResult("xx")
//                .proposerCode("x")
//                .build());


        return McR.success(approveRecordDao.findAllByOrderCodeIn(Arrays.asList("2023-03-014")));
    }
}
