package com.malk.xintianlong.schedule;

import com.malk.service.dingtalk.DDClient;
import com.malk.service.dingtalk.DDClient_Event;
import com.malk.service.dingtalk.DDClient_Workflow;
import com.malk.xintianlong.repository.dao.HSApproveRecordDao;
import com.malk.xintianlong.repository.entity.HSApproveRecordPo;
import com.malk.xintianlong.service.HSService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @EnableScheduling 开启定时任务 [配置参考McScheduleTask]
 */
@Slf4j
@Configuration
@EnableScheduling
@ConditionalOnProperty(name = {"spel.scheduling"})
public class HSScheduleTask {

    @Autowired
    private DDClient ddClient;

    @Autowired
    private DDClient_Workflow ddClient_workflow;

    @Autowired
    private HSApproveRecordDao hsApproveRecordDao;

    @Autowired
    private HSService hsService;

    /**
     * 同步审批中数据, [0...5分钟/次]  - 未废弃[环思接口请求可能会存在超时情况]
     */
    @Scheduled(cron = "0 0/5 * * * ?")
    public void syncYSB2COrderToDingTalkApprove() {
        try {

            log.info("###### [HS]开始同步审批中记录 ######");
            List<HSApproveRecordPo> list = hsApproveRecordDao.findAllByApproveResult("process");
            log.info("同步数量, {}", list.size());
            for (HSApproveRecordPo po : list) {
                Map rsp = ddClient_workflow.getProcessInstanceId(ddClient.getAccessToken(), po.getProcessInstanceId());
                if (Arrays.asList("RUNNING", "NEW").contains(rsp.get("status"))) {
                    continue;
                }
                if (Arrays.asList("TERMINATED", "CANCELED").contains(rsp.get("status"))) {
                    hsService.syncApproveResult2Hs(po.getProcessInstanceId(), "terminated");
                }
                if (StringUtils.isNotBlank(String.valueOf(rsp.get("result")))) {
                    hsService.syncApproveResult2Hs(po.getProcessInstanceId(), "agree".equals(String.valueOf(rsp.get("result"))) ? "agree" : "refuse");
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e); // 记录错误日志
        }
        log.info("###### [HS]同步审批中记录结束 ######");
    }

    @Autowired
    private DDClient_Event ddClient_event;

    /**
     * 同步钉钉推送失败记录: 推送失败列表, 获取后记录会被清空
     */
    @Scheduled(cron = "0 0/30 7-23 * * ?")
    public void syncDingTalkFailedList() {
        try {
            ddClient_event.syncFailedList(ddClient.getAccessToken());
        } catch (Exception e) {
            // 记录错误信息
            e.printStackTrace();
        }
    }
}