package com.malk.xintianlong.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.malk.Util.UtilHttp;
import com.malk.Util.UtilMap;
import com.malk.Util.UtilServlet;
import com.malk.server.common.McException;
import com.malk.xintianlong.repository.dao.HSApproveRecordDao;
import com.malk.xintianlong.repository.entity.HSApproveRecordPo;
import com.malk.xintianlong.server.HSConf;
import com.malk.xintianlong.service.HSService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.Map;

@Service
@Slf4j
public class HSServiceImpl implements HSService {

    @Autowired
    private HSApproveRecordDao xtlApproveRecordDao;

    /**
     * 校验调用授权
     */
    @Override
    public void validateAuthorization(HttpServletRequest request, String orderCode, String corpType) {
        Map headers = UtilServlet.getHeaders(request);
        log.info("校验调用授权, {}", headers);
        McException.assertAccessException(!headers.containsKey("authorization") || !headers.get("authorization").equals(HSConf.ACCESS_TOKEN_DD), "未获得接口授权");
        HSApproveRecordPo po = xtlApproveRecordDao.findByOrderCodeAndApproveResultAndCorpType(orderCode, "process", corpType);
        McException.assertAccessException(ObjectUtil.isNotNull(po), "单据已在审批中, 请勿重复提交!");
    }

    /**
     * 同步环思审批结果
     */
    @Override
    public Map syncApproveResult2Hs(String processInstanceId, String approveResult) {
        HSApproveRecordPo po = xtlApproveRecordDao.findByProcessInstanceId(processInstanceId);
        Map item = UtilMap.map("sBillNo, iStatus, sRemark", po.getOrderCode(), HSConf.APPROVE_RESULT_HS.get(approveResult), "");
        Map xmlData = UtilMap.map("type, items", po.getFormType(), Arrays.asList(item));
        Map param = UtilMap.map("Token, DataXml", HSConf.ACCESS_TOKE_HS, JSON.toJSONString(xmlData));
        Map<String, Map> rsp = UtilHttp.doSoap(HSConf.CALLBACK_URL_HS.get(po.getCorpType()), param, "DataSync", "http://tempuri.org/");
        Map hsRsp = rsp.get("soap:Body");
        log.info("调用环思回调结果, {}, ", hsRsp);
        xtlApproveRecordDao.updateApproveResult(processInstanceId, approveResult, JSON.toJSONString(hsRsp));
        return hsRsp;
    }
}
